;(function () {
	var root = this;	
	var NewsApplication = root.NewsApplication;
	var Views = NewsApplication.Views = {};
	
	Views.MapView = Marionette.CompositeView.extend({
		id: 'mapCanvas',
		
		template: function () {
			return _.template(NewsApplication.Templates.mapsTemplate);
		},
		
		initialize: function (locations) {
			this.parseLocations(locations);
		},
		
		parseLocations: function (locations) {
			if (!locations) return;
			var locs = [];
			for (var i = 0; i < locations.length; i++) {
				if (locations[i]) {
					locs.push(locations[i]);
				}
			}
			this.locations = locs;
		},
			
		createMap: function () {			
			var mapOptions = {
				zoom: 10,
	            mapTypeId: google.maps.MapTypeId.ROADMAP
	        };
			
			var createMarker = function (lat, long, map, title) {
				var pos = new google.maps.LatLng(lat, long);
				var marker = new google.maps.Marker({
					position: pos,
					map: map,
					title: title
				});
				return marker;
			};
			
	        var domElement = this.$el;
	        var map = new google.maps.Map(domElement.get(0), mapOptions);
	        
	        var createMarkers = function (list) {
	        	if (!list[0]) { return; }
	        	var pos = _.first(list).split(',');
	        	createMarker(pos[0], pos[1], map, '');
	        	return createMarkers(_.rest(list))
	        };
	        createMarkers(_.flatten(this.locations));
	        
	        if(navigator.geolocation) {
	            navigator.geolocation.getCurrentPosition(function(position) {
	            	createMarker(position.coords.latitude, position.coords.longitude, map, 'Your position');
	            	map.setCenter(new google.maps.LatLng(position.coords.latitude, position.coords.longitude));
	            }, function() {
	            	this.handleNoGeolocation(true);
	            });
	          } else {
	            this.handleNoGeolocation(false);
	          }
	    },
	    
	    handleNoGeolocation: function (errorFlag) {
	    	  if (errorFlag) {
	    	    var content = 'Error: The Geolocation service failed.';
	    	  } else {
	    	    var content = 'Error: Your browser doesn\'t support geolocation.';
	    	  }
	    },
	    
	    render: function () {
	    	this.$el.html(this.template);
	    	this.createMap();
	    	return this;
	    }
	});

	Views.Header = Marionette.CompositeView.extend({
		className: 'header-wrapper',
		
		template: function () {
			return _.template(NewsApplication.Templates.headerStuffTemplate);
		},

		events: {
			'click #mapBtn': 'showOnMap',
			'click #settings': 'showSettings',
			'click .filterBtn': 'hideSettings',
			'click #five': 'fiveClicked',
			'click #ten': 'tenClicked',
			'click #fifteen': 'fifteenClicked',
			'click #technology': 'technology',
			'click #entertainment': 'entertainment',
			'click #lifestyle': 'lifestyle',
			'click #news': 'news',
			'click #sport': 'sport',
			'click #economy': 'economy'
		},
		
		economy: function () {
			Backbone.history.navigate('economy', {
				trigger: true
			});
		},
		
		sport: function () {
			Backbone.history.navigate('sport', {
				trigger: true
			});
		},
		
		news: function () {
			Backbone.history.navigate('news', {
				trigger: true
			});
		},
		
		technology: function () {
			Backbone.history.navigate('technology', {
				trigger: true
			});
		},
		
		entertainment: function () {
			Backbone.history.navigate('entertainment', {
				trigger: true
			});
		},
		
		lifestyle: function () {
			Backbone.history.navigate('lifestyle', {
				trigger: true
			});
		},
		
		fiveClicked: function () {
			NewsApplication.Vent.trigger('range', 5);
		},
		
		tenClicked: function () {
			NewsApplication.Vent.trigger('range', 10);
		},
		
		fifteenClicked: function () {
			NewsApplication.Vent.trigger('range', 15);
		},

		showSettings: function(event){
			$('#nav').slideToggle();
		},
		
		hideSettings: function(event){
			$('#nav').slideToggle();
		},
		
		showOnMap: function(event) {
			console.log('map selected');
			NewsApplication.Vent.trigger('map');
		}
	});

	Views.News = Marionette.ItemView.extend({
		className: 'news-list grid_12',
		
		template: function (model) {
			return _.template(NewsApplication.Templates.newsTemplate, {
				publisher: model.publisher,
				published: this.getFormattedDate(model),
				title: model.title,
				lead_text: model.lead_text
			});
		},
		
		getFormattedDate: function (model) {
			var date = new Date(model.published);
			var day = date.getDate();
			var month = date.getMonth();
			var year = date.getFullYear();
			return day + '.' + month + '.' + year;
		},

		events: {
			'click': 'storySelected'
		},

		initialize: function () {
			this.bind('change', this.render, this);			
		},

		destroy: function () {
			this.model.destroy();
		},

		storySelected: function () {
			NewsApplication.Vent.trigger('storySelected', this.model);
		},
		
		render: function () {
			var model = this.model.toJSON();
			this.$el.html(this.template(model));
			
			if (model.cat[0] === 'NEWS') {
				this.$('.story').addClass('news');
			}
			if (model.cat[0] === 'SPORT') {
				this.$('.story').addClass('sport');	
			}
			if (model.cat[0] === 'ENTERTAINMENT') {
				this.$('.story').addClass('entertainment');	
			}
			if (model.cat[0] === 'ECONOMY') {
				this.$('.story').addClass('economy');
			}
			if (model.cat[0] === 'TECHNOLOGY') {
				this.$('.story').addClass('technology');
			}
			if (model.cat[0] === 'LIFESTYLE') {
				this.$('.story').addClass('lifestyle');
			}
			return this;
		}
	});

	Views.NewsExtended = Marionette.CompositeView.extend({
		className: 'extendedNews row',
		
		template: function (model) {
			return _.template(NewsApplication.Templates.newsExtendedTemplate, {
				title: model.title,
				lead: model.lead_text,
				img: model.image_url[0],
				text: model.text
			});
		},

		events: {
			'click img': 'maps'
		},

		maps: function() {
			NewsApplication.Vent.trigger('maps', this.model.toJSON().location);
		}
	});

	Views.NewsList = Marionette.CompositeView.extend({		
		itemViewContainer: '#stories-list',
		itemView: Views.News,
		
		template: function () {
			return _.template(NewsApplication.Templates.storiesTemplate);
		},
		
		initialize: function (options) {
			this.listenTo(this.collection, 'sync', this.update);
			this.collection.fetch({
				reset: true
			});
		},
		
		update: function () {
			this.render();
		}
	});
}).call(this);